package com.global.iop.api;

package .tmp;

import com.global.iop.api.IopClient;
import com.global.iop.api.IopRequest;
import com.global.iop.api.IopResponse;
import com.global.iop.util.ApiException;
import com.global.iop.util.IopHashMap;
import com.global.iop.util.IopLogger;
import com.global.iop.util.IopUtils;
import com.global.iop.util.RequestContext;
import com.global.iop.util.WebUtils;
import com.global.iop.util.json.JSONValidatingReader;
import java.io.IOException;
import java.net.Proxy;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class IopClientImpl implements IopClient {
  protected String serverUrl;
  
  protected String appKey;
  
  protected String appSecret;
  
  protected String signMethod = "sha256";
  
  protected int connectTimeout = 15000;
  
  protected int readTimeout = 30000;
  
  protected boolean useGzipEncoding = true;
  
  protected Proxy proxy;
  
  protected String sdkVersion = "iop-sdk-java-20181207";
  
  protected String logLevel = "ERROR";
  
  public IopClientImpl(String serverUrl, String appKey, String appSecret) {
    this.appKey = appKey;
    this.appSecret = appSecret;
    this.serverUrl = serverUrl;
  }
  
  public IopClientImpl(String serverUrl, String appKey, String appSecret, int connectTimeout, int readTimeout) {
    this(serverUrl, appKey, appSecret);
    this.connectTimeout = connectTimeout;
    this.readTimeout = readTimeout;
  }
  
  public IopResponse execute(IopRequest request) throws ApiException {
    return execute(request, null);
  }
  
  public IopResponse execute(IopRequest request, String accessToken) throws ApiException {
    return doExecute(request, accessToken);
  }
  
  private IopResponse doExecute(IopRequest request, String accessToken) throws ApiException {
    long start = System.currentTimeMillis();
    RequestContext requestContext = new RequestContext();
    IopHashMap bizParams = new IopHashMap((request.getApiParams() != null) ? (Map)request.getApiParams() : new HashMap<Object, Object>());
    requestContext.setQueryParams(bizParams);
    requestContext.setApiName(request.getApiName());
    IopHashMap commonParams = new IopHashMap();
    commonParams.put("app_key", this.appKey);
    Long timestamp = request.getTimestamp();
    if (timestamp == null)
      timestamp = Long.valueOf(System.currentTimeMillis()); 
    commonParams.put("timestamp", new Date(timestamp.longValue()));
    commonParams.put("sign_method", this.signMethod);
    commonParams.put("access_token", accessToken);
    commonParams.put("partner_id", this.sdkVersion);
    if (isDebugEnabled())
      commonParams.put("debug", Boolean.valueOf(true)); 
    requestContext.setCommonParams(commonParams);
    try {
      commonParams.put("sign", IopUtils.signApiRequest(requestContext, this.appSecret, this.signMethod));
      String rpcUrl = WebUtils.buildRestUrl(this.serverUrl, request.getApiName());
      String urlQuery = WebUtils.buildQuery((Map)requestContext.getCommonParams(), "UTF-8");
      String fullUrl = WebUtils.buildRequestUrl(rpcUrl, new String[] { urlQuery });
      String rsp = null;
      if (this.useGzipEncoding)
        request.addHeaderParameter("Accept-Encoding", "gzip"); 
      if (request.getFileParams() != null) {
        rsp = WebUtils.doPost(fullUrl, (Map)bizParams, request.getFileParams(), request.getHeaderParams(), "UTF-8", this.connectTimeout, this.readTimeout);
      } else if (request.getHttpMethod().equals("POST")) {
        rsp = WebUtils.doPost(fullUrl, (Map)bizParams, request.getHeaderParams(), "UTF-8", this.connectTimeout, this.readTimeout, this.proxy);
      } else {
        rsp = WebUtils.doGet(fullUrl, (Map)bizParams, request.getHeaderParams(), this.connectTimeout, this.readTimeout, "UTF-8", this.proxy);
      } 
      requestContext.setResponseBody(rsp);
    } catch (IOException e) {
      IopLogger.write(this.appKey, this.sdkVersion, request.getApiName(), this.serverUrl, requestContext.getAllParams(), System.currentTimeMillis() - start, e.toString());
      throw new ApiException(e);
    } catch (Exception e) {
      IopLogger.write(this.appKey, this.sdkVersion, request.getApiName(), this.serverUrl, requestContext.getAllParams(), System.currentTimeMillis() - start, e.toString());
      throw new ApiException(e);
    } 
    IopResponse response = parseResponse(requestContext.getResponseBody());
    if (!response.isSuccess()) {
      IopLogger.write(this.appKey, this.sdkVersion, request.getApiName(), this.serverUrl, requestContext.getAllParams(), System.currentTimeMillis() - start, response.getBody());
    } else if (isDebugEnabled() || isInfoEnabled()) {
      IopLogger.write(this.appKey, this.sdkVersion, request.getApiName(), this.serverUrl, requestContext.getAllParams(), System.currentTimeMillis() - start, "");
    } 
    return response;
  }
  
  private IopResponse parseResponse(String jsonRsp) {
    JSONValidatingReader jSONValidatingReader = new JSONValidatingReader();
    Map<?, ?> root = (Map<?, ?>)jSONValidatingReader.read(jsonRsp);
    IopResponse response = new IopResponse();
    response.setType((String)root.get("type"));
    response.setCode((String)root.get("code"));
    response.setMessage((String)root.get("message"));
    response.setRequestId((String)root.get("request_id"));
    response.setBody(jsonRsp);
    return response;
  }
  
  public void setNeedEnableLogger(boolean needEnableLogger) {
    IopLogger.setNeedEnableLogger(needEnableLogger);
  }
  
  public void setIgnoreSSLCheck(boolean ignore) {
    WebUtils.setIgnoreSSLCheck(ignore);
  }
  
  public void setUseGzipEncoding(boolean useGzipEncoding) {
    this.useGzipEncoding = useGzipEncoding;
  }
  
  public void setConnectTimeout(int connectTimeout) {
    this.connectTimeout = connectTimeout;
  }
  
  public void setReadTimeout(int readTimeout) {
    this.readTimeout = readTimeout;
  }
  
  public void setSignMethod(String signMethod) {
    this.signMethod = signMethod;
  }
  
  public void setProxy(Proxy proxy) {
    this.proxy = proxy;
  }
  
  public void setLogLevel(String logLevel) {
    this.logLevel = logLevel;
  }
  
  public boolean isDebugEnabled() {
    return this.logLevel.equals("DEBUG");
  }
  
  public boolean isInfoEnabled() {
    return this.logLevel.equals("INFO");
  }
  
  public boolean isErrorEnabled() {
    return this.logLevel.equals("ERROR");
  }
}
